from .async_client import AsyncClient



class AsyncStockProvider(AsyncClient):
    def __init__(self, client, ws, callback):
        super().__init__(client, ws)
        self.start_listening(callback)




import asyncio
from datetime import datetime

# provider has flag for request / response 

class KrossProvider:
    def __init__(self, socket, node):
        self.sock = socket
        self.node = node
        self.pending = False
        self.subscribe_count = 0
        self.write_lock = asyncio.Lock()

    async def send_request(self, query):
        async with self.write_lock:
            print('send request done')
            await self.sock.send(query.SerializeToString())

    async def listen(self):
        while True:
            try:
                recv_data = await self.sock.recv()
            except Exception as e:
                print(e)
                break



import uuid
import asyncio


class AsyncClient:
    def __init__(self, client, ws):
        self.client = client
        self.ws = ws
        self.listen_task = None

    def _fill_uuid(self, msg):
        msg_id = uuid.uuid4().int
        msg.uuid_h = msg_id >> 64
        msg.uuid_l = msg_id & (1 << 64) -1

    def start_listening(self, callback):
        self.listen_task = asyncio.create_task(self.listening(callback))

    async def listening(self, callback):
        while True:
            try:
                print('start listening')
                recv_data = await self.ws.recv()
                callback(recv_data, socket=self.ws)
            finally:
                pass



from .async_client import AsyncClient


class AsyncStockHandler(AsyncClient):
    def __init__(self, client, ws):
        super().__init__(client, ws)

    async def get_day_data(self, query):
        self._fill_uuid(query)

        await self.ws.send(query.SerializeToString())
        print('waiting recv')
        data = await self.ws.recv()
        return data



import asyncio

from .protocol import stock_pb2
from . import providerdict


class KrossClient:
    def __init__(self, socket, node):
        self.sock = socket
        self.node = node
        self.write_lock = asyncio.Lock()

    async def send_object(self, data):
        async with self.write_lock:
            await self.sock.send(data.SerializeToString())

    async def send_response(self, query, result_code):
        result = stock_pb2.StockResult()
        result.uuid_h = query.uuid_h
        result.uuid_l = query.uuid_l
        result.code = result_code
        await self.send_object(result)

    async def listen(self):
        while True:
            try:
                recv_data = await self.sock.recv()
                print('recv msg', len(recv_data))
                query = stock_pb2.StockQuery()
                query.ParseFromString(recv_data)
                provider = await providerdict.find_provider(self.node.vendor,
                              self.node.channel_type,
                              self.node.service)

                print(provider)
                if provider is None:
                    await self.send_response(query, stock_pb2.NO_PROVIDER)
                else:
                    await provider.send_request(query)

            except Exception as e:
                print(e)
                break
        print('quit listening...')



import asyncio
import collections
from .protocol import tunnel_pb2


dict_lock = asyncio.Lock()
vendor_dict = dict()


# depth 1: vendor, depth 2: channel, depth 3: service
def _add_to_dict(provider):
    print('add_to_dict', provider)
    vendor = provider.node.vendor
    channel = provider.node.channel_type
    service = provider.node.service
    if vendor not in vendor_dict:
        vendor_dict[vendor] = dict()

    if channel not in vendor_dict[vendor]:
        vendor_dict[vendor][channel] = dict()

    if service not in vendor_dict[vendor][channel]:
        vendor_dict[vendor][channel][service] = collections.deque()

    vendor_dict[vendor][channel][service].append(provider)


async def add_provider(provider):
    async with dict_lock:
        _add_to_dict(provider)


def _clear_provider():
    global vendor_dict
    vendor_dict = dict()


async def find_rqrs_provider(vendor, channel, service):
    try:
        providers = vendor_dict[vendor][channel][service]
    except KeyError:
        return None
    
    available_provider = None
    for provider in providers:
        if not provider.pending:
            provider.pending = True
            available_provider = provider
            providers.rotate(-1)
            break
    
    return available_provider


async def find_provider(vendor, channel, service):
    available_provider = None
    if channel == tunnel_pb2.REQ_RES:
        available_provider = await find_rqrs_provider(vendor, channel, service)
    elif channel == tunnel_pb2.PUBLISH_SUBSCRIBE:
        pass

    return available_provider



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tunnel.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='tunnel.proto',
  package='tunnel_api',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0ctunnel.proto\x12\ntunnel_api\"\x99\x01\n\x04Node\x12\x0e\n\x06vendor\x18\x01 \x01(\t\x12-\n\x0c\x63hannel_type\x18\x02 \x01(\x0e\x32\x17.tunnel_api.ChannelType\x12\x1e\n\x04role\x18\x03 \x01(\x0e\x32\x10.tunnel_api.Role\x12\x0c\n\x04name\x18\x04 \x01(\t\x12$\n\x07service\x18\x05 \x01(\x0e\x32\x13.tunnel_api.Service\"\x8b\x01\n\x06Result\x12\x32\n\x0bresult_code\x18\x01 \x01(\x0e\x32\x1d.tunnel_api.Result.ResultType\"M\n\nResultType\x12\x06\n\x02OK\x10\x00\x12\x11\n\rUNKNOWN_ERROR\x10\x01\x12\x11\n\rNOT_AVAILABLE\x10\x02\x12\x11\n\rMESSAGE_ERROR\x10\x03*1\n\x0b\x43hannelType\x12\x0b\n\x07REQ_RES\x10\x00\x12\x15\n\x11PUBLISH_SUBSCRIBE\x10\x01*%\n\x07Service\x12\t\n\x05STOCK\x10\x00\x12\x0f\n\x0bSTOCK_ORDER\x10\x01* \n\x04Role\x12\n\n\x06\x43LIENT\x10\x00\x12\x0c\n\x08PROVIDER\x10\x01\x62\x06proto3'
)

_CHANNELTYPE = _descriptor.EnumDescriptor(
  name='ChannelType',
  full_name='tunnel_api.ChannelType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='REQ_RES', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PUBLISH_SUBSCRIBE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=326,
  serialized_end=375,
)
_sym_db.RegisterEnumDescriptor(_CHANNELTYPE)

ChannelType = enum_type_wrapper.EnumTypeWrapper(_CHANNELTYPE)
_SERVICE = _descriptor.EnumDescriptor(
  name='Service',
  full_name='tunnel_api.Service',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='STOCK', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='STOCK_ORDER', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=377,
  serialized_end=414,
)
_sym_db.RegisterEnumDescriptor(_SERVICE)

Service = enum_type_wrapper.EnumTypeWrapper(_SERVICE)
_ROLE = _descriptor.EnumDescriptor(
  name='Role',
  full_name='tunnel_api.Role',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='CLIENT', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PROVIDER', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=416,
  serialized_end=448,
)
_sym_db.RegisterEnumDescriptor(_ROLE)

Role = enum_type_wrapper.EnumTypeWrapper(_ROLE)
REQ_RES = 0
PUBLISH_SUBSCRIBE = 1
STOCK = 0
STOCK_ORDER = 1
CLIENT = 0
PROVIDER = 1


_RESULT_RESULTTYPE = _descriptor.EnumDescriptor(
  name='ResultType',
  full_name='tunnel_api.Result.ResultType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='OK', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN_ERROR', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NOT_AVAILABLE', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MESSAGE_ERROR', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=247,
  serialized_end=324,
)
_sym_db.RegisterEnumDescriptor(_RESULT_RESULTTYPE)


_NODE = _descriptor.Descriptor(
  name='Node',
  full_name='tunnel_api.Node',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='vendor', full_name='tunnel_api.Node.vendor', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='channel_type', full_name='tunnel_api.Node.channel_type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='role', full_name='tunnel_api.Node.role', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='tunnel_api.Node.name', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='service', full_name='tunnel_api.Node.service', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=29,
  serialized_end=182,
)


_RESULT = _descriptor.Descriptor(
  name='Result',
  full_name='tunnel_api.Result',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='result_code', full_name='tunnel_api.Result.result_code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _RESULT_RESULTTYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=185,
  serialized_end=324,
)

_NODE.fields_by_name['channel_type'].enum_type = _CHANNELTYPE
_NODE.fields_by_name['role'].enum_type = _ROLE
_NODE.fields_by_name['service'].enum_type = _SERVICE
_RESULT.fields_by_name['result_code'].enum_type = _RESULT_RESULTTYPE
_RESULT_RESULTTYPE.containing_type = _RESULT
DESCRIPTOR.message_types_by_name['Node'] = _NODE
DESCRIPTOR.message_types_by_name['Result'] = _RESULT
DESCRIPTOR.enum_types_by_name['ChannelType'] = _CHANNELTYPE
DESCRIPTOR.enum_types_by_name['Service'] = _SERVICE
DESCRIPTOR.enum_types_by_name['Role'] = _ROLE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Node = _reflection.GeneratedProtocolMessageType('Node', (_message.Message,), {
  'DESCRIPTOR' : _NODE,
  '__module__' : 'tunnel_pb2'
  # @@protoc_insertion_point(class_scope:tunnel_api.Node)
  })
_sym_db.RegisterMessage(Node)

Result = _reflection.GeneratedProtocolMessageType('Result', (_message.Message,), {
  'DESCRIPTOR' : _RESULT,
  '__module__' : 'tunnel_pb2'
  # @@protoc_insertion_point(class_scope:tunnel_api.Result)
  })
_sym_db.RegisterMessage(Result)


# @@protoc_insertion_point(module_scope)



import asyncio
import websockets
from websockets.client import WebSocketClientProtocol

from .protocol import tunnel_pb2
from .stock.async_stocksubscriber import AsyncStockSubscriber
from .stock.async_stockhandler import AsyncStockHandler
from .stock.async_client import AsyncClient
from .stock.async_stockprovider import AsyncStockProvider

from . import exceptions


def get_result_code(data) -> tunnel_pb2.Result:
    r = tunnel_pb2.Result()
    r.ParseFromString(data)
    return r.result_code


async def handshake(role, vendor, channel_type, service, options
) -> (WebSocketClientProtocol, int, tunnel_pb2.Node):
    try:
        ws = await websockets.connect('ws://localhost:8765')
    except:
        raise

    node = tunnel_pb2.Node()
    node.vendor = vendor
    node.role = role
    node.channel_type = channel_type
    node.service = service
    node.name = '' if 'name' not in options else options['name']

    await ws.send(node.SerializeToString())
    result_code = get_result_code(await ws.recv())

    return ws, result_code, node


async def stock_service(vendor, channel_type, **kwargs) -> AsyncClient:
    try:
        ws, result_code, node = await handshake(tunnel_pb2.CLIENT,
                                                vendor,
                                                channel_type,
                                                tunnel_pb2.STOCK, kwargs)
    except:
        raise
    else:
        if result_code == tunnel_pb2.Result.OK:
            if channel_type == tunnel_pb2.REQ_RES:
                return AsyncStockHandler(node, ws)
            elif channel_type == tunnel_pb2.PUBLISH_SUBSCRIBE:
                try:
                    return AsyncStockSubscriber(node,
                                                ws,
                                                kwargs['callback'])
                except KeyError:
                    return AsyncStockSubscriber(node, ws, None)
        else:
            raise exceptions.KrossClientException('handshake failed')
    

async def stock_provider_service(vendor, channel_type, **kwargs) -> AsyncStockProvider:
    try:
        ws, result_code, node = await handshake(tunnel_pb2.PROVIDER,
                                                vendor,
                                                channel_type,
                                                tunnel_pb2.STOCK,
                                                kwargs)
    except:
        raise
    else:
        if result_code == tunnel_pb2.Result.OK: 
            try:
                return AsyncStockProvider(node,
                                          ws,
                                          kwargs['callback'])
            except:
                pass

    raise exceptions.KrossClientException('handshake failed')


def run(entry_point) -> None:
    asyncio.get_event_loop().run_until_complete(entry_point)
    asyncio.get_event_loop().run_forever()



from .async_client import AsyncClient


class AsyncStockSubscriber(AsyncClient):
    def __init__(self, client, ws, callback):
        super().__init__(client, ws)
        self.start_listening(callback)


    async def subscribe_stock(self, symbol):
        pass



import pathlib
import re

import setuptools


root_dir = pathlib.Path(__file__).parent

description = 'KrossTrading Core Package'

long_description = 'KrossTrading core package for multi-vendor, clients'
#long_description = (root_dir / 'README.md').read_text(encoding='utf-8')

# PyPI disables the "raw" directive.
#long_description = re.sub(
#    r"^\.\. raw:: html.*?^(?=\w)",
#    "",
#    long_description,
#    flags=re.DOTALL | re.MULTILINE,
#)

exec((root_dir / 'ktrade' / 'version.py').read_text(encoding='utf-8'))

packages = ['ktrade', 'ktrade/protocol', 'ktrade/stock']

setuptools.setup(
    name='ktrade',
    version=version,
    description=description,
    long_description=long_description,
    url='https://github.com/krosstrading/ktrade',
    author='KrossTrading',
    author_email='contacts@krosstrading.com',
    license='Apache License 2.0',
    classifiers=[
        'Development Status :: 1 - Planning',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    package_dir = {'': '.'},
    #package_data
    packages=packages,
    include_package_data=True,
    zip_safe=False,
    python_requires='>=3.7',
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    #test_loader='pytest',
)




import asyncio
import websockets

import google.protobuf.message as message
from .protocol import tunnel_pb2
from . import provider
from . import client
from . import providerdict

""" for debugging websockets
import logging
logger = logging.getLogger('websockets')
logger.setLevel(logging.DEBUG)
logger.addHandler(logging.StreamHandler())
"""


async def _add_node(node, ws) -> None:
    node_wrapper = None

    if node.role == tunnel_pb2.PROVIDER:
        node_wrapper = provider.KrossProvider(ws, node)
        await providerdict.add_provider(node_wrapper)
    elif node.role == tunnel_pb2.CLIENT:
        node_wrapper = client.KrossClient(ws, node)

    if node_wrapper is not None:
        await ws.send(_create_result(tunnel_pb2.Result.OK).SerializeToString())
        await node_wrapper.listen()
    else:
        await ws.send(_create_result(tunnel_pb2.Result.NOT_AVAILABLE).SerializeToString())


def _create_result(code) -> tunnel_pb2.Result:
    r = tunnel_pb2.Result()
    r.result_code = code
    return r


async def accept(ws, path) -> None:
    print('node connected')
    recv_data = await ws.recv()
    node = tunnel_pb2.Node()
    try:
        node.ParseFromString(recv_data)
    except message.DecodeError:
        await ws.send(_create_result(tunnel_pb2.Result.MESSAGE_ERROR).SerializeToString())
    else:
        await _add_node(node, ws)


if __name__ == '__main__':
    start_server = websockets.serve(accept, "localhost", 8765, 
                                    ping_timeout=5, ping_interval=5, compression=None)

    asyncio.get_event_loop().run_until_complete(start_server)
    asyncio.get_event_loop().run_forever()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: stock.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='stock.proto',
  package='stock_api',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0bstock.proto\x12\tstock_api\x1a\x1fgoogle/protobuf/timestamp.proto\"\x84\x01\n\rStockDayQuery\x12\x0c\n\x04\x63ode\x18\x01 \x01(\t\x12\x31\n\rfrom_datetime\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x32\n\x0euntil_datetime\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"d\n\nStockQuery\x12\x0e\n\x06uuid_h\x18\x01 \x01(\x04\x12\x0e\n\x06uuid_l\x18\x02 \x01(\x04\x12-\n\tday_query\x18\x03 \x01(\x0b\x32\x18.stock_api.StockDayQueryH\x00\x42\x07\n\x05query\"R\n\x0bStockResult\x12\x0e\n\x06uuid_h\x18\x01 \x01(\x04\x12\x0e\n\x06uuid_l\x18\x02 \x01(\x04\x12#\n\x04\x63ode\x18\x03 \x01(\x0e\x32\x15.stock_api.ResultCode*;\n\nResultCode\x12\x06\n\x02OK\x10\x00\x12\x0f\n\x0bNO_PROVIDER\x10\x01\x12\x14\n\x10RESPONSE_TIMEOUT\x10\x02\x62\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])

_RESULTCODE = _descriptor.EnumDescriptor(
  name='ResultCode',
  full_name='stock_api.ResultCode',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='OK', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NO_PROVIDER', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RESPONSE_TIMEOUT', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=380,
  serialized_end=439,
)
_sym_db.RegisterEnumDescriptor(_RESULTCODE)

ResultCode = enum_type_wrapper.EnumTypeWrapper(_RESULTCODE)
OK = 0
NO_PROVIDER = 1
RESPONSE_TIMEOUT = 2



_STOCKDAYQUERY = _descriptor.Descriptor(
  name='StockDayQuery',
  full_name='stock_api.StockDayQuery',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='code', full_name='stock_api.StockDayQuery.code', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='from_datetime', full_name='stock_api.StockDayQuery.from_datetime', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='until_datetime', full_name='stock_api.StockDayQuery.until_datetime', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=60,
  serialized_end=192,
)


_STOCKQUERY = _descriptor.Descriptor(
  name='StockQuery',
  full_name='stock_api.StockQuery',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='uuid_h', full_name='stock_api.StockQuery.uuid_h', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uuid_l', full_name='stock_api.StockQuery.uuid_l', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='day_query', full_name='stock_api.StockQuery.day_query', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='query', full_name='stock_api.StockQuery.query',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=194,
  serialized_end=294,
)


_STOCKRESULT = _descriptor.Descriptor(
  name='StockResult',
  full_name='stock_api.StockResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='uuid_h', full_name='stock_api.StockResult.uuid_h', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uuid_l', full_name='stock_api.StockResult.uuid_l', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='code', full_name='stock_api.StockResult.code', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=296,
  serialized_end=378,
)

_STOCKDAYQUERY.fields_by_name['from_datetime'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_STOCKDAYQUERY.fields_by_name['until_datetime'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_STOCKQUERY.fields_by_name['day_query'].message_type = _STOCKDAYQUERY
_STOCKQUERY.oneofs_by_name['query'].fields.append(
  _STOCKQUERY.fields_by_name['day_query'])
_STOCKQUERY.fields_by_name['day_query'].containing_oneof = _STOCKQUERY.oneofs_by_name['query']
_STOCKRESULT.fields_by_name['code'].enum_type = _RESULTCODE
DESCRIPTOR.message_types_by_name['StockDayQuery'] = _STOCKDAYQUERY
DESCRIPTOR.message_types_by_name['StockQuery'] = _STOCKQUERY
DESCRIPTOR.message_types_by_name['StockResult'] = _STOCKRESULT
DESCRIPTOR.enum_types_by_name['ResultCode'] = _RESULTCODE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

StockDayQuery = _reflection.GeneratedProtocolMessageType('StockDayQuery', (_message.Message,), {
  'DESCRIPTOR' : _STOCKDAYQUERY,
  '__module__' : 'stock_pb2'
  # @@protoc_insertion_point(class_scope:stock_api.StockDayQuery)
  })
_sym_db.RegisterMessage(StockDayQuery)

StockQuery = _reflection.GeneratedProtocolMessageType('StockQuery', (_message.Message,), {
  'DESCRIPTOR' : _STOCKQUERY,
  '__module__' : 'stock_pb2'
  # @@protoc_insertion_point(class_scope:stock_api.StockQuery)
  })
_sym_db.RegisterMessage(StockQuery)

StockResult = _reflection.GeneratedProtocolMessageType('StockResult', (_message.Message,), {
  'DESCRIPTOR' : _STOCKRESULT,
  '__module__' : 'stock_pb2'
  # @@protoc_insertion_point(class_scope:stock_api.StockResult)
  })
_sym_db.RegisterMessage(StockResult)


# @@protoc_insertion_point(module_scope)
